local addon, private = ...
local mod = LibStub("AceAddon-3.0"):GetAddon(addon)

local dropItem = function(self)
  local t, id, link = GetCursorInfo()
  if t == "item" and link then
    local itemName, itemLink, quality, level, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice, classId, subClassId = GetItemInfo(link)
    if CursorCanGoInSlot(self.info.slot_id) or (classId == LE_ITEM_CLASS_GEM and subClassId == LE_ITEM_GEM_ARTIFACTRELIC and self.info.artifact_gem_id) then
      self.setItem(link)
      ClearCursor()
      Update()
    end
  end
end

mod.frameSlots = {}

-- The comparison UI frame
local SimItemCompareFrame = CreateFrame("Frame", "SimItemCompareFrame", UIParent, "UIPanelDialogTemplate")
mod.UI = SimItemCompareFrame
SimItemCompareFrame.title:SetText("SimCraft Comparison Builder")
SimItemCompareFrame.title:SetTextColor(255, 255, 255)

SimItemCompareFrame:SetToplevel(true)
SimItemCompareFrame:SetFrameStrata("HIGH")
SimItemCompareFrame:Hide()
SimItemCompareFrame:EnableMouse(true)
SimItemCompareFrame:SetSize(600, 425)
SimItemCompareFrame:SetPoint("TOPLEFT", CharacterFrame, "TOPRIGHT", 15, 0)
SimItemCompareFrame:SetMovable(true)
SimItemCompareFrame:SetScript("OnDragStart", function()
  SimItemCompareFrame:StartMoving()
end)
SimItemCompareFrame:SetScript("OnDragStop", function()
  SimItemCompareFrame:StopMovingOrSizing()
end)
tinsert(UISpecialFrames, "SimItemCompareFrame")
UIPanelWindows["SimItemCompareFrame"] =  { area = "left", pushable = 3, whileDead = 1}

local SimItemCopyFrame = CreateFrame("Frame", "SimItemCopyFrame", SimItemCompareFrame, "UIPanelDialogTemplate")
SimItemCopyFrame:Hide()
SimItemCopyFrame:SetBackdropColor(0, 0, 0, 1)
SimItemCopyFrame:SetSize(600, 200)
SimItemCopyFrame:SetPoint("TOPLEFT", SimItemCompareFrame, "BOTTOMLEFT")
SimItemCopyFrame:SetPoint("TOPRIGHT", SimItemCompareFrame, "BOTTOMRIGHT")
SimItemCopyFrame:SetFrameStrata("HIGH")
SimItemCopyFrame:EnableMouse(true)
SimItemCopyFrame:SetToplevel(true)
SimItemCopyFrame.title:SetText("SimCraft - Comparison Profile")
mod.CopyFrame = SimItemCopyFrame

local SimItemCopyFrameScroll = CreateFrame("ScrollFrame", nil, SimItemCopyFrame, "UIPanelScrollFrameTemplate")
SimItemCopyFrameScroll:SetPoint("TOPLEFT", SimItemCopyFrame, 15, -35)
SimItemCopyFrameScroll:SetPoint("BOTTOMRIGHT", SimItemCopyFrame, -35, 15)
SimItemCopyFrameScroll:SetFrameStrata("HIGH")
SimItemCopyFrameScroll:EnableMouse(true)
SimItemCopyFrameScroll:SetToplevel(true)

local editBox = CreateFrame("EditBox", nil, SimItemCopyFrameScroll)
mod.editBox = editBox
editBox:SetMultiLine(true)
editBox:SetMaxLetters(99999)
editBox:EnableMouse(true)
editBox:SetAutoFocus(false)
editBox:SetFontObject(ChatFontNormal)
editBox:SetWidth(600)
editBox:SetHeight(270)
editBox:SetPoint("TOPLEFT", SimItemCopyFrame, "TOPLEFT")
editBox:SetPoint("BOTTOMRIGHT", SimItemCopyFrame, "BOTTOMRIGHT")
editBox:SetScript("OnEscapePressed", function() editBox:ClearFocus()  end)
editBox:SetFrameStrata("HIGH")
editBox:SetToplevel(true)
SimItemCopyFrameScroll:SetScrollChild(editBox)

for i, slotGroup in ipairs(mod.SLOT_GROUPS) do
  for j, slot in ipairs(slotGroup) do
    local name = "SimItemCompareFrameButton_" .. tostring(i) .. "_" .. tostring(j)
    local f = CreateFrame("Button", name, SimItemCompareFrame, "ItemButtonTemplate")
    tinsert(mod.frameSlots, f)
    f.info = slot
    f:EnableMouse(true)
    f:SetWidth(40)
    f:SetHeight(40)
    f:RegisterForClicks("LeftButtonDown", "RightButtonUp")
    local x = (i - 1) * 200 + 15
    local y = -45 * (j - 1) - 35
    f:SetPoint("TOPLEFT", x, y)

    f.title = f:CreateFontString(nil, nil, "SystemFont_Outline_Small")
    f.title:SetPoint("LEFT", f, "RIGHT", 4, 0)
    f.title:SetWordWrap(true)
    f.title:SetWidth(135)
    f.title:SetJustifyH("LEFT")

    f:SetScript("OnReceiveDrag", dropItem)
    f:SetScript("OnClick", function(self, button)
      if button == "RightButton" and not GetCursorInfo() then
        f.setItem(nil)
      else
        dropItem(self)
      end
    end)

    f.setItem = function(item)
      local itemName, itemLink, quality, level, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice, classId, subClassId
      local r, g, b, hex = 255, 255, 255, 255
      if item then
        itemName, itemLink, quality, level, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice, classId, subClassId = GetItemInfo(item)
        r, g, b, hex = GetItemQualityColor(quality)
        f.title:SetText("|cffffffff" .. slot.name .. "|h\n|c" .. hex .. itemName .. " (" .. tostring(level) .. ")")
        _G[name .. "IconTexture"]:SetTexture(texture)
      else
        _G[name .. "IconTexture"]:SetTexture(nil)
        f.title:SetText("|cffffffff" .. slot.name)
      end
      f.itemLink = itemLink
    end

    f:SetScript("OnEnter", function()
      if f.itemLink then
        GameTooltip:SetOwner(f, "ANCHOR_CURSOR")
        GameTooltip:SetHyperlink(f.itemLink)
        GameTooltip:Show()
      end
    end)

    f:SetScript("OnLeave", function()
      GameTooltip:Hide()
    end)
  end
end

do
  local wantsLoad = false
  ArtifactFrame:HookScript("OnShow", function()
    if wantsLoad then
      wantsLoad = false
      mod:ScheduleTimer(function()
        HideUIPanel(ArtifactFrame)
      end, 0.1)
      mod.UI:Show()
      mod:copySlotsFromEquipped()
    end
  end)

  mod.loadWindow = function()
    if mod.UI:IsVisible() then
      wantsLoad = false
      mod.UI:Hide()
    else
      wantsLoad = true
      SocketInventoryItem(getSocketSlotID())
    end
  end

  -- Button attached to the character frame
  local SimItemCompareButton = CreateFrame("Button", "SimBuilderLaunchButton", CharacterFrame, "UIGoldBorderButtonTemplate")
  SimItemCompareButton:SetSize(44, 18)
  SimItemCompareButton:SetText("SimC")
  SimItemCompareButton:SetPoint("BOTTOMRIGHT", CharacterFrame, "BOTTOMRIGHT", -10, 5)
  SimItemCompareButton:SetScript("OnClick", mod.loadWindow)
end